"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandHelper = void 0;
var path = require("path");
var CommandHelper = (function () {
    function CommandHelper() {
        this.cliOptions = [];
        this.cliArguments = [];
        this.parsedArguments = {};
        this.parsedOptions = {};
        this.argvParsed = false;
    }
    CommandHelper.prototype.addArgument = function (name, isRequired, help) {
        if (this.cliArguments.slice(-1).length && this.cliArguments.slice(-1)[0].isRequired)
            throw new Error("Required argument '" + name + "' must be placed in front of optional arguments");
        this.cliArguments.push({
            name: name,
            isRequired: isRequired,
            help: help,
        });
    };
    CommandHelper.prototype.addOption = function (name, short, type, def, help) {
        if (this.cliArguments.length)
            throw new Error("Options must be defined before all arguments");
        this.cliOptions.push({
            name: name,
            short: short,
            type: type,
            def: def,
            help: help,
        });
    };
    CommandHelper.prototype.getUsage = function () {
        var str = "Usage:\n  "
            + path.basename(process.argv[1]) + " " + path.basename(process.argv[3]) + " ";
        if (this.cliOptions.length)
            str += "[options] ";
        str += this.cliArguments.map(function (a) { return a.isRequired ? "<" + a.name + ">" : "[" + a.name + "]"; }).join(" ");
        if (this.cliOptions.length) {
            str += "\noptions:\n"
                + this.cliOptions.map(function (o) { return "  -" + o.short + ", --" + o.name + ": " + o.help + " [" + (o.def === true ? 'true' : o.def === false ? 'false' : o.def) + "]\n"; }).join('');
        }
        if (this.cliArguments.length) {
            str += "\narguments:\n"
                + this.cliArguments.map(function (a) { return "  " + a.name + " - " + a.help + "\n"; }).join('');
        }
        return str;
    };
    CommandHelper.prototype.getArgumentValue = function (name) {
        this.parseArgv();
        return this.parsedArguments[name];
    };
    CommandHelper.prototype.getOptionValue = function (name) {
        this.parseArgv();
        return this.parsedOptions[name];
    };
    CommandHelper.prototype.parseArgv = function () {
        var _this = this;
        var _a, _b, _c, _d, _e, _f;
        if (this.argvParsed)
            return;
        this.argvParsed = true;
        this.parsedArguments = {};
        this.parsedOptions = {};
        var args = process.argv.slice(3);
        this.cliOptions.forEach(function (o) { return _this.parsedOptions[o.name] = o.def; });
        var re = /^((?<short>-\w)|(?<long>--\w+))$/;
        var _loop_1 = function () {
            var str = (_a = args.shift()) !== null && _a !== void 0 ? _a : "";
            var matches = (_c = (_b = re.exec(str)) === null || _b === void 0 ? void 0 : _b.groups) !== null && _c !== void 0 ? _c : {};
            var option = this_1.cliOptions.filter(function (o) { return matches.short ? o.short === matches.short.slice(1) : o.name === matches.long.slice(2); })[0];
            if (!option)
                throw Error("Unknown option '" + str + "'");
            if (option.type === "bool") {
                this_1.parsedOptions[option.name] = true;
            }
            else if (option.type === "required") {
                this_1.parsedOptions[option.name] = (_d = args.shift()) !== null && _d !== void 0 ? _d : option.def;
            }
            else if (!re.test(args[0])) {
                this_1.parsedOptions[option.name] = (_e = args.shift()) !== null && _e !== void 0 ? _e : option.def;
            }
        };
        var this_1 = this;
        while (args[0] && re.test(args[0])) {
            _loop_1();
        }
        for (var _i = 0, _g = this.cliArguments; _i < _g.length; _i++) {
            var arg = _g[_i];
            if (arg.isRequired) {
                var v = args.shift();
                if (!v)
                    throw new Error("Argument '" + arg.name + "' is required");
                else
                    this.parsedArguments[arg.name] = v;
            }
            else {
                if (args.length > 0)
                    this.parsedArguments[arg.name] = (_f = args.shift()) !== null && _f !== void 0 ? _f : "";
            }
        }
    };
    return CommandHelper;
}());
exports.CommandHelper = CommandHelper;
