"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Help = void 0;
var path = require("path");
var CommandHelper_1 = require("./CommandHelper");
var Help = (function () {
    function Help() {
        this._commands = {};
        this.helper = new CommandHelper_1.CommandHelper();
    }
    Object.defineProperty(Help.prototype, "commands", {
        set: function (commands) {
            this._commands = commands;
        },
        enumerable: false,
        configurable: true
    });
    Help.prototype.description = function () {
        return "Show help";
    };
    Help.prototype.help = function () {
        return "The help command can be used to get instructions on how to run any of the configured commands.\n\n" +
            this.helper.getUsage();
    };
    Help.prototype.configure = function () {
        this.helper.addArgument('command', false, 'Command name to show help for');
    };
    Help.prototype.execute = function () {
        var command;
        var commandName = this.helper.getArgumentValue('command');
        if (commandName) {
            if (this._commands.hasOwnProperty(commandName)) {
                command = new this._commands[commandName];
                command.configure();
                console.log(command.help());
            }
            else {
                throw new Error("Unknown command '" + commandName + "'");
            }
        }
        else {
            console.log("List of commands:");
            for (var commandName_1 in this._commands) {
                if (this._commands.hasOwnProperty(commandName_1)) {
                    var command_1 = new this._commands[commandName_1];
                    console.log("  " + commandName_1 + " - " + command_1.description());
                }
            }
            console.log("\nTo get detailed help for a command, run '" + path.basename(process.argv[1]) + " help < command name >'");
        }
        return 0;
    };
    return Help;
}());
exports.Help = Help;
